@@ -26,13 +26,11 @@
 
 /******************************************************************************/
 
-{
-// >>>>> start of local scope
+import webext from './webext.js';
+import { ubolog } from './console.js';
 
-/******************************************************************************/
 /******************************************************************************/
 
-const browser = self.browser;
 const manifest = browser.runtime.getManifest();
 
 vAPI.cantWebsocket =
@@ -42,7 +40,7 @@ vAPI.cantWebsocket =
 vAPI.canWASM = vAPI.webextFlavor.soup.has('chromium') === false;
 if ( vAPI.canWASM === false ) {
     const csp = manifest.content_security_policy;
-    vAPI.canWASM = csp !== undefined && csp.indexOf("'wasm-eval'") !== -1;
+    vAPI.canWASM = csp !== undefined && csp.indexOf("'unsafe-eval'") !== -1;
 }
 
 vAPI.supportsUserStylesheets = vAPI.webextFlavor.soup.has('user_stylesheet');
@@ -56,16 +54,6 @@ window.addEventListener('webextFlavor', function() {
 
 /******************************************************************************/
 
-vAPI.randomToken = function() {
-    const n = Math.random();
-    return String.fromCharCode(n * 26 + 97) +
-        Math.floor(
-            (0.25 + n * 0.75) * Number.MAX_SAFE_INTEGER
-        ).toString(36).slice(-8);
-};
-
-/******************************************************************************/
-
 vAPI.app = {
     name: manifest.name.replace(/ dev\w+ build/, ''),
     version: (( ) => {
@@ -125,82 +113,10 @@ vAPI.browserSettings = (( ) => {
     if ( bp instanceof Object === false ) { return; }
 
     return {
-        // Whether the WebRTC-related privacy API is crashy is an open question
-        // only for Chromium proper (because it can be compiled without the
-        // WebRTC feature): hence avoid overhead of the evaluation (which uses
-        // an iframe) for platforms where it's a non-issue.
-        // https://github.com/uBlockOrigin/uBlock-issues/issues/9
-        //   Some Chromium builds are made to look like a Chrome build.
-        webRTCSupported: vAPI.webextFlavor.soup.has('chromium') === false || undefined,
-
-        // Calling with `true` means IP address leak is not prevented.
-        // https://github.com/gorhill/uBlock/issues/533
-        //   We must first check wether this Chromium-based browser was compiled
-        //   with WebRTC support. To do this, we use an iframe, this way the
-        //   empty RTCPeerConnection object we create to test for support will
-        //   be properly garbage collected. This prevents issues such as
-        //   a computer unable to enter into sleep mode, as reported in the
-        //   Chrome store:
-        // https://github.com/gorhill/uBlock/issues/533#issuecomment-167931681
-        setWebrtcIPAddress: function(setting) {
-            // We don't know yet whether this browser supports WebRTC: find out.
-            if ( this.webRTCSupported === undefined ) {
-                // If asked to leave WebRTC setting alone at this point in the
-                // code, this means we never grabbed the setting in the first
-                // place.
-                if ( setting ) { return; }
-                this.webRTCSupported = { setting: setting };
-                let iframe = document.createElement('iframe');
-                const messageHandler = ev => {
-                    if ( ev.origin !== self.location.origin ) { return; }
-                    window.removeEventListener('message', messageHandler);
-                    const setting = this.webRTCSupported.setting;
-                    this.webRTCSupported = ev.data === 'webRTCSupported';
-                    this.setWebrtcIPAddress(setting);
-                    iframe.parentNode.removeChild(iframe);
-                    iframe = null;
-                };
-                window.addEventListener('message', messageHandler);
-                iframe.src = 'is-webrtc-supported.html';
-                document.body.appendChild(iframe);
-                return;
-            }
-
-            // We are waiting for a response from our iframe. This makes the code
-            // safe to re-entrancy.
-            if ( typeof this.webRTCSupported === 'object' ) {
-                this.webRTCSupported.setting = setting;
-                return;
-            }
-
-            // https://github.com/gorhill/uBlock/issues/533
-            // WebRTC not supported: `webRTCMultipleRoutesEnabled` can NOT be
-            // safely accessed. Accessing the property will cause full browser
-            // crash.
-            if ( this.webRTCSupported !== true ) { return; }
-
-            const bpn = bp.network;
-
-            if ( setting ) {
-                bpn.webRTCIPHandlingPolicy.clear({
-                    scope: 'regular',
-                });
-            } else {
-                // https://github.com/uBlockOrigin/uAssets/issues/333#issuecomment-289426678
-                //   Leverage virtuous side-effect of strictest setting.
-                // https://github.com/gorhill/uBlock/issues/3009
-                //   Firefox currently works differently, use
-                //   `default_public_interface_only` for now.
-                // https://developer.mozilla.org/en-US/docs/Mozilla/Add-ons/WebExtensions/API/privacy/network#Browser_compatibility
-                //   Firefox 70+ supports `disable_non_proxied_udp`
-                const value =
-                    vAPI.webextFlavor.soup.has('firefox') === false ||
-                    vAPI.webextFlavor.major < 70
-                        ? 'default_public_interface_only'
-                        : 'disable_non_proxied_udp';
-                bpn.webRTCIPHandlingPolicy.set({ value, scope: 'regular' });
-            }
-        },
+        // https://github.com/uBlockOrigin/uBlock-issues/issues/1723#issuecomment-919913361
+        canLeakLocalIPAddresses:
+            vAPI.webextFlavor.soup.has('firefox') &&
+            vAPI.webextFlavor.soup.has('mobile'),
 
         set: function(details) {
             for ( const setting in details ) {
@@ -237,7 +153,17 @@ vAPI.browserSettings = (( ) => {
                     break;
 
                 case 'webrtcIPAddress':
-                    this.setWebrtcIPAddress(!!details[setting]);
+                    if ( this.canLeakLocalIPAddresses === false ) { return; }
+                    if ( !!details[setting] ) {
+                        bp.network.webRTCIPHandlingPolicy.clear({
+                            scope: 'regular',
+                        });
+                    } else {
+                        bp.network.webRTCIPHandlingPolicy.set({
+                            value: 'default_public_interface_only',
+                            scope: 'regular'
+                        });
+                    }
                     break;
 
                 default:
@@ -476,16 +402,6 @@ vAPI.Tabs = class {
             return;
         }
 
-        // https://github.com/gorhill/uBlock/issues/3053#issuecomment-332276818
-        //   Do not try to lookup uBO's own pages with FF 55 or less.
-        if (
-            vAPI.webextFlavor.soup.has('firefox') &&
-            vAPI.webextFlavor.major < 56
-        ) {
-            this.create(targetURL, details);
-            return;
-        }
-
         // https://developer.mozilla.org/en-US/docs/Mozilla/Add-ons/WebExtensions/API/tabs/query#Parameters
         //   "Note that fragment identifiers are not matched."
         //   Fragment identifiers ARE matched -- we need to remove the fragment.
@@ -600,9 +516,12 @@ vAPI.Tabs = class {
     }
 
     onUpdatedHandler(tabId, changeInfo, tab) {
+        // Ignore uninteresting update events
+        const { status = '', title = '', url = '' } = changeInfo;
+        if ( status === '' && title === '' && url === '' ) { return; }
         // https://github.com/gorhill/uBlock/issues/3073
         //   Fall back to `tab.url` when `changeInfo.url` is not set.
-        if ( typeof changeInfo.url !== 'string' ) {
+        if ( url === '' ) {
             changeInfo.url = tab && tab.url;
         }
         if ( changeInfo.url ) {
@@ -1143,7 +1062,7 @@ vAPI.messaging = {
         }
 
         // Auxiliary process to main process: no handler
-        log.info(
+        ubolog(
             `vAPI.messaging.onPortMessage > unhandled request: ${JSON.stringify(request.msg)}`,
             request
         );
@@ -1319,50 +1238,6 @@ vAPI.Net = class {
     canSuspend() {
         return false;
     }
-    async benchmark() {
-        if ( typeof µBlock !== 'object' ) { return; }
-        const requests = await µBlock.loadBenchmarkDataset();
-        if ( Array.isArray(requests) === false || requests.length === 0 ) {
-            console.info('No requests found to benchmark');
-            return;
-        }
-        const mappedTypes = new Map([
-            [ 'document', 'main_frame' ],
-            [ 'subdocument', 'sub_frame' ],
-        ]);
-        console.info('vAPI.net.onBeforeSuspendableRequest()...');
-        const t0 = self.performance.now();
-        const promises = [];
-        const details = {
-            documentUrl: '',
-            tabId: -1,
-            parentFrameId: -1,
-            frameId: 0,
-            type: '',
-            url: '',
-        };
-        for ( const request of requests ) {
-            details.documentUrl = request.frameUrl;
-            details.tabId = -1;
-            details.parentFrameId = -1;
-            details.frameId = 0;
-            details.type = mappedTypes.get(request.cpt) || request.cpt;
-            details.url = request.url;
-            if ( details.type === 'main_frame' ) { continue; }
-            promises.push(this.onBeforeSuspendableRequest(details));
-        }
-        return Promise.all(promises).then(results => {
-            let blockCount = 0;
-            for ( const r of results ) {
-                if ( r !== undefined ) { blockCount += 1; }
-            }
-            const t1 = self.performance.now();
-            const dur = t1 - t0;
-            console.info(`Evaluated ${requests.length} requests in ${dur.toFixed(0)} ms`);
-            console.info(`\tBlocked ${blockCount} requests`);
-            console.info(`\tAverage: ${(dur / requests.length).toFixed(3)} ms per request`);
-        });
-    }
 };
 
 /******************************************************************************/
@@ -1719,9 +1594,3 @@ vAPI.cloud = (( ) => {
 })();
 
 /******************************************************************************/
-/******************************************************************************/
-
-// <<<<< end of local scope
-}
-
-/******************************************************************************/
