@@ -25,7 +25,7 @@
 
 /******************************************************************************/
 
-(( ) => {
+import './codemirror/ubo-static-filtering.js';
 
 /******************************************************************************/
 
@@ -352,5 +352,3 @@ cmEditor.on('changes', userFiltersChanged);
 CodeMirror.commands.save = applyChanges;
 
 /******************************************************************************/
-
-})();
