@@ -25,11 +25,13 @@
 
 /******************************************************************************/
 
-(( ) => {
+{
+// >>>>> start of local scope
 
 /******************************************************************************/
 
 const lastUpdateTemplateString = vAPI.i18n('3pLastUpdate');
+const obsoleteTemplateString = vAPI.i18n('3pExternalListObsolete');
 const reValidExternalList = /^[a-z-]+:\/\/(?:\S+\/\S*|\/\S+)/m;
 
 let listDetails = {};
@@ -118,6 +120,7 @@ const renderFilterLists = function(soft) {
             } else {
                 li.classList.remove('mustread');
             }
+            li.classList.toggle('isDefault', entry.isDefault === true);
             li.classList.toggle('unused', hideUnused && !on);
         }
         // https://github.com/gorhill/uBlock/issues/1429
@@ -141,14 +144,22 @@ const renderFilterLists = function(soft) {
         );
         li.classList.toggle('failed', asset.error !== undefined);
         li.classList.toggle('obsolete', asset.obsolete === true);
+        const lastUpdateString = lastUpdateTemplateString.replace(
+            '{{ago}}',
+            renderElapsedTimeToString(asset.writeTime || 0)
+        );
+        if ( asset.obsolete === true ) {
+            let title = obsoleteTemplateString;
+            if ( asset.cached && asset.writeTime !== 0 ) {
+                title += '\n' + lastUpdateString;
+            }
+            li.querySelector('.status.obsolete').setAttribute('title', title);
+        }
         if ( asset.cached === true ) {
             li.classList.add('cached');
             li.querySelector('.status.cache').setAttribute(
                 'title',
-                lastUpdateTemplateString.replace(
-                    '{{ago}}',
-                    renderElapsedTimeToString(asset.writeTime)
-                )
+                lastUpdateString
             );
         } else {
             li.classList.remove('cached');
@@ -678,7 +689,7 @@ uDom('#ignoreGenericCosmeticFilters').on('change', onFilteringSettingsChanged);
 uDom('#buttonApply').on('click', ( ) => { buttonApplyHandler(); });
 uDom('#buttonUpdate').on('click', ( ) => { buttonUpdateHandler(); });
 uDom('#buttonPurgeAll').on('click', ev => {
-    buttonPurgeAllHandler(ev.ctrlKey && ev.shiftKey);
+    buttonPurgeAllHandler(ev.shiftKey);
 });
 uDom('#lists').on('change', '.listEntry input', onListsetChanged);
 uDom('#lists').on('click', '.listEntry .remove', onRemoveExternalList);
@@ -696,5 +707,6 @@ renderFilterLists();
 
 /******************************************************************************/
 
-})();
+// <<<<< end of local scope
+}
 
