@@ -23,7 +23,9 @@
 
 /******************************************************************************/
 
-µBlock.assets = (( ) => {
+import cacheStorage from './cachestorage.js';
+import logger from './logger.js';
+import µb from './background.js';
 
 /******************************************************************************/
 
@@ -31,7 +33,7 @@ const reIsExternalPath = /^(?:[a-z-]+):\/\//;
 const reIsUserAsset = /^user-/;
 const errorCantConnectTo = vAPI.i18n('errorCantConnectTo');
 
-const api = {};
+const assets = {};
 
 // A hint for various pieces of code to take measures if possible to save
 // bandwidth of remote servers.
@@ -41,13 +43,13 @@ let remoteServerFriendly = false;
 
 const observers = [];
 
-api.addObserver = function(observer) {
+assets.addObserver = function(observer) {
     if ( observers.indexOf(observer) === -1 ) {
         observers.push(observer);
     }
 };
 
-api.removeObserver = function(observer) {
+assets.removeObserver = function(observer) {
     let pos;
     while ( (pos = observers.indexOf(observer)) !== -1 ) {
         observers.splice(pos, 1);
@@ -65,11 +67,11 @@ const fireNotification = function(topic, details) {
 
 /******************************************************************************/
 
-api.fetch = function(url, options = {}) {
+assets.fetch = function(url, options = {}) {
     return new Promise((resolve, reject) => {
     // Start of executor
 
-    const timeoutAfter = µBlock.hiddenSettings.assetFetchTimeout * 1000 || 30000;
+    const timeoutAfter = µb.hiddenSettings.assetFetchTimeout * 1000 || 30000;
     const xhr = new XMLHttpRequest();
     let contentLoaded = 0;
     let timeoutTimer;
@@ -86,7 +88,7 @@ api.fetch = function(url, options = {}) {
     };
 
     const fail = function(details, msg) {
-        µBlock.logger.writeOne({
+        logger.writeOne({
             realm: 'message',
             type: 'error',
             text: msg,
@@ -154,7 +156,7 @@ api.fetch = function(url, options = {}) {
 
 /******************************************************************************/
 
-api.fetchText = async function(url) {
+assets.fetchText = async function(url) {
     const isExternal = reIsExternalPath.test(url);
     let actualUrl = isExternal ? url : vAPI.getURL(url);
 
@@ -171,7 +173,7 @@ api.fetchText = async function(url) {
     // servers.
     if ( isExternal && remoteServerFriendly !== true ) {
         const cacheBypassToken =
-            µBlock.hiddenSettings.updateAssetBypassBrowserCache
+            µb.hiddenSettings.updateAssetBypassBrowserCache
                 ? Math.floor(Date.now() /    1000) % 86413
                 : Math.floor(Date.now() / 3600000) %    13;
         const queryValue = `_=${cacheBypassToken}`;
@@ -185,7 +187,7 @@ api.fetchText = async function(url) {
 
     let details = { content: '' };
     try {
-        details = await api.fetch(actualUrl);
+        details = await assets.fetch(actualUrl);
 
         // Consider an empty result to be an error
         if ( stringIsNotEmpty(details.content) === false ) {
@@ -199,6 +201,14 @@ api.fetchText = async function(url) {
         if ( text.startsWith('<') && text.endsWith('>') ) {
             details.content = '';
         }
+
+        // Important: Non empty text resource must always end with a newline
+        if (
+            details.content.length !== 0 &&
+            details.content.endsWith('\n') === false
+        ) {
+            details.content += '\n';
+        }
     } catch(ex) {
         details = ex;
     }
@@ -215,10 +225,10 @@ api.fetchText = async function(url) {
 // https://github.com/gorhill/uBlock/issues/3331
 //   Support the seamless loading of sublists.
 
-api.fetchFilterList = async function(mainlistURL) {
+assets.fetchFilterList = async function(mainlistURL) {
     const toParsedURL = url => {
         try {
-            return new URL(url);
+            return new URL(url.trim());
         } catch (ex) {
         }
     };
@@ -245,9 +255,11 @@ api.fetchFilterList = async function(mainlistURL) {
     const sublistURLs = new Set();
 
     // https://github.com/uBlockOrigin/uBlock-issues/issues/1113
+    //   Process only `!#include` directives which are not excluded by an
+    //   `!#if` directive.
     const processIncludeDirectives = function(results) {
         const out = [];
-        const reInclude = /^!#include +(\S+)/gm;
+        const reInclude = /^!#include +(\S+)[^\n\r]*(?:[\n\r]+|$)/gm;
         for ( const result of results ) {
             if ( typeof result === 'string' ) {
                 out.push(result);
@@ -255,7 +267,7 @@ api.fetchFilterList = async function(mainlistURL) {
             }
             if ( result instanceof Object === false ) { continue; }
             const content = result.content;
-            const slices = µBlock.preparseDirectives.split(content);
+            const slices = µb.preparseDirectives.split(content);
             for ( let i = 0, n = slices.length - 1; i < n; i++ ) {
                 const slice = content.slice(slices[i+0], slices[i+1]);
                 if ( (i & 1) !== 0 ) {
@@ -272,14 +284,14 @@ api.fetchFilterList = async function(mainlistURL) {
                     // Compute nested list path relative to parent list path
                     const pos = result.url.lastIndexOf('/');
                     if ( pos === -1 ) { continue; }
-                    const subURL = result.url.slice(0, pos + 1) + match[1];
+                    const subURL = result.url.slice(0, pos + 1) + match[1].trim();
                     if ( sublistURLs.has(subURL) ) { continue; }
                     sublistURLs.add(subURL);
                     out.push(
                         slice.slice(lastIndex, match.index + match[0].length),
-                        `! >>>>>>>> ${subURL}`,
-                        api.fetchText(subURL),
-                        `! <<<<<<<< ${subURL}`
+                        `! >>>>>>>> ${subURL}\n`,
+                        assets.fetchText(subURL),
+                        `! <<<<<<<< ${subURL}\n`
                     );
                     lastIndex = reInclude.lastIndex;
                 }
@@ -297,15 +309,24 @@ api.fetchFilterList = async function(mainlistURL) {
     let allParts = [
         this.fetchText(mainlistURL)
     ];
-    for (;;) {
-        allParts = processIncludeDirectives(await Promise.all(allParts));
-        if ( allParts.every(v => typeof v === 'string') ) { break; }
-    }
+    // Abort processing `include` directives if at least one included sublist
+    // can't be fetched.
+    do {
+        allParts = await Promise.all(allParts);
+        const part = allParts.find(part => {
+            return typeof part === 'object' && part.error !== undefined;
+        });
+        if ( part !== undefined ) {
+            return { url: mainlistURL, content: '', error: part.error };
+        }
+        allParts = processIncludeDirectives(allParts);
+    } while ( allParts.some(part => typeof part !== 'string') );
+    // If we reach this point, this means all fetches were successful.
     return {
         url: mainlistURL,
         content: allParts.length === 1
             ? allParts[0]
-            : allParts.map(s => s.trim()).filter(s => s !== '').join('\n') + '\n'
+            : allParts.join('') + '\n'
     };
 };
 
@@ -327,7 +348,7 @@ let assetSourceRegistryPromise,
 
 const getAssetSourceRegistry = function() {
     if ( assetSourceRegistryPromise === undefined ) {
-        assetSourceRegistryPromise = µBlock.cacheStorage.get(
+        assetSourceRegistryPromise = cacheStorage.get(
             'assetSourceRegistry'
         ).then(bin => {
             if (
@@ -337,12 +358,12 @@ const getAssetSourceRegistry = function() {
                 assetSourceRegistry = bin.assetSourceRegistry;
                 return assetSourceRegistry;
             }
-            return api.fetchText(
-                µBlock.assetsBootstrapLocation || 'assets/assets.json'
+            return assets.fetchText(
+                µb.assetsBootstrapLocation || 'assets/assets.json'
             ).then(details => {
                 return details.content !== ''
                     ? details
-                    : api.fetchText('assets/assets.json');
+                    : assets.fetchText('assets/assets.json');
             }).then(details => {
                 updateAssetSourceRegistry(details.content, true);
                 return assetSourceRegistry;
@@ -399,7 +420,7 @@ const saveAssetSourceRegistry = (( ) => {
     let timer;
     const save = function() {
         timer = undefined;
-        µBlock.cacheStorage.set({ assetSourceRegistry });
+        cacheStorage.set({ assetSourceRegistry });
     };
     return function(lazily) {
         if ( timer !== undefined ) {
@@ -445,13 +466,13 @@ const updateAssetSourceRegistry = function(json, silent) {
     saveAssetSourceRegistry();
 };
 
-api.registerAssetSource = async function(assetKey, details) {
+assets.registerAssetSource = async function(assetKey, details) {
     await getAssetSourceRegistry();
     registerAssetSource(assetKey, details);
     saveAssetSourceRegistry(true);
 };
 
-api.unregisterAssetSource = async function(assetKey) {
+assets.unregisterAssetSource = async function(assetKey) {
     await getAssetSourceRegistry();
     unregisterAssetSource(assetKey);
     saveAssetSourceRegistry(true);
@@ -470,7 +491,7 @@ let assetCacheRegistry = {};
 
 const getAssetCacheRegistry = function() {
     if ( assetCacheRegistryPromise === undefined ) {
-        assetCacheRegistryPromise = µBlock.cacheStorage.get(
+        assetCacheRegistryPromise = cacheStorage.get(
             'assetCacheRegistry'
         ).then(bin => {
             if (
@@ -504,7 +525,7 @@ const saveAssetCacheRegistry = (( ) => {
     let timer;
     const save = function() {
         timer = undefined;
-        µBlock.cacheStorage.set({ assetCacheRegistry });
+        cacheStorage.set({ assetCacheRegistry });
     };
     return function(lazily) {
         if ( timer !== undefined ) { clearTimeout(timer); }
@@ -528,7 +549,7 @@ const assetCacheRead = async function(assetKey, updateReadTime = false) {
 
     const [ , bin ] = await Promise.all([
         getAssetCacheRegistry(),
-        µBlock.cacheStorage.get(internalKey),
+        cacheStorage.get(internalKey),
     ]);
     if (
         bin instanceof Object === false ||
@@ -574,7 +595,7 @@ const assetCacheWrite = async function(assetKey, details) {
     if ( typeof options.url === 'string' ) {
         entry.remoteURL = options.url;
     }
-    µBlock.cacheStorage.set({
+    cacheStorage.set({
         assetCacheRegistry,
         [`cache/${assetKey}`]: content
     });
@@ -604,8 +625,8 @@ const assetCacheRemove = async function(pattern) {
     }
     if ( removedContent.length !== 0 ) {
         await Promise.all([
-            µBlock.cacheStorage.remove(removedContent),
-            µBlock.cacheStorage.set({ assetCacheRegistry }),
+            cacheStorage.remove(removedContent),
+            cacheStorage.set({ assetCacheRegistry }),
         ]);
     }
     for ( let i = 0; i < removedEntries.length; i++ ) {
@@ -639,7 +660,7 @@ const assetCacheMarkAsDirty = async function(pattern, exclude) {
         mustSave = true;
     }
     if ( mustSave ) {
-        µBlock.cacheStorage.set({ assetCacheRegistry });
+        cacheStorage.set({ assetCacheRegistry });
     }
 };
 
@@ -689,8 +710,8 @@ const saveUserAsset = function(assetKey, content) {
 
 /******************************************************************************/
 
-api.get = async function(assetKey, options = {}) {
-    if ( assetKey === µBlock.userFiltersPath ) {
+assets.get = async function(assetKey, options = {}) {
+    if ( assetKey === µb.userFiltersPath ) {
         return readUserAsset(assetKey);
     }
 
@@ -730,14 +751,20 @@ api.get = async function(assetKey, options = {}) {
 
     const assetRegistry = await getAssetSourceRegistry();
     assetDetails = assetRegistry[assetKey] || {};
-    let contentURLs = [];
+    const contentURLs = [];
     if ( typeof assetDetails.contentURL === 'string' ) {
-        contentURLs = [ assetDetails.contentURL ];
+        contentURLs.push(assetDetails.contentURL);
     } else if ( Array.isArray(assetDetails.contentURL) ) {
-        contentURLs = assetDetails.contentURL.slice(0);
+        contentURLs.push(...assetDetails.contentURL);
     } else if ( reIsExternalPath.test(assetKey) ) {
         assetDetails.content = 'filters';
-        contentURLs = [ assetKey ];
+        contentURLs.push(assetKey);
+    }
+
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/1566#issuecomment-826473517
+    //   Use CDN URLs as fall back URLs.
+    if ( Array.isArray(assetDetails.cdnURLs) ) {
+        contentURLs.push(...assetDetails.cdnURLs);
     }
 
     for ( const contentURL of contentURLs ) {
@@ -745,8 +772,8 @@ api.get = async function(assetKey, options = {}) {
             continue;
         }
         const details = assetDetails.content === 'filters'
-            ? await api.fetchFilterList(contentURL)
-            : await api.fetchText(contentURL);
+            ? await assets.fetchFilterList(contentURL)
+            : await assets.fetchText(contentURL);
         if ( details.content === '' ) { continue; }
         if ( reIsExternalPath.test(contentURL) && options.dontCache !== true ) {
             assetCacheWrite(assetKey, {
@@ -776,32 +803,39 @@ const getRemote = async function(assetKey) {
         return details;
     };
 
-    let contentURLs = [];
+    const contentURLs = [];
     if ( typeof assetDetails.contentURL === 'string' ) {
-        contentURLs = [ assetDetails.contentURL ];
+        contentURLs.push(assetDetails.contentURL);
     } else if ( Array.isArray(assetDetails.contentURL) ) {
-        contentURLs = assetDetails.contentURL.slice(0);
+        contentURLs.push(...assetDetails.contentURL);
     }
 
     // If asked to be gentle on remote servers, favour using dedicated CDN
     // servers. If more than one CDN server is present, randomly shuffle the
     // set of servers so as to spread the bandwidth burden.
-    if ( remoteServerFriendly && Array.isArray(assetDetails.cdnURLs) ) {
+    //
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/1566#issuecomment-826473517
+    //   In case of manual update, use CDNs URLs as fall back URLs.
+    if ( Array.isArray(assetDetails.cdnURLs) ) {
         const cdnURLs = assetDetails.cdnURLs.slice();
         for ( let i = 0, n = cdnURLs.length; i < n; i++ ) {
             const j = Math.floor(Math.random() * n);
             if ( j === i ) { continue; }
             [ cdnURLs[j], cdnURLs[i] ] = [ cdnURLs[i], cdnURLs[j] ];
         }
-        contentURLs.unshift(...cdnURLs);
+        if ( remoteServerFriendly ) {
+            contentURLs.unshift(...cdnURLs);
+        } else {
+            contentURLs.push(...cdnURLs);
+        }
     }
 
     for ( const contentURL of contentURLs ) {
         if ( reIsExternalPath.test(contentURL) === false ) { continue; }
 
         const result = assetDetails.content === 'filters'
-            ? await api.fetchFilterList(contentURL)
-            : await api.fetchText(contentURL);
+            ? await assets.fetchFilterList(contentURL)
+            : await assets.fetchText(contentURL);
 
         // Failure
         if ( stringIsNotEmpty(result.content) === false ) {
@@ -829,7 +863,7 @@ const getRemote = async function(assetKey) {
 
 /******************************************************************************/
 
-api.put = async function(assetKey, content) {
+assets.put = async function(assetKey, content) {
     return reIsUserAsset.test(assetKey)
         ? await saveUserAsset(assetKey, content)
         : await assetCacheWrite(assetKey, content);
@@ -837,7 +871,7 @@ api.put = async function(assetKey, content) {
 
 /******************************************************************************/
 
-api.metadata = async function() {
+assets.metadata = async function() {
     await Promise.all([
         getAssetSourceRegistry(),
         getAssetCacheRegistry(),
@@ -849,6 +883,15 @@ api.metadata = async function() {
     for ( const assetKey in assetDict ) {
         const assetEntry = assetDict[assetKey];
         const cacheEntry = cacheDict[assetKey];
+        if (
+            assetEntry.content === 'filters' &&
+            assetEntry.external !== true
+        ) {
+            assetEntry.isDefault =
+                assetEntry.off === undefined ||
+                assetEntry.off === true &&
+                    µb.listMatchesEnvironment(assetEntry);
+        }
         if ( cacheEntry ) {
             assetEntry.cached = true;
             assetEntry.writeTime = cacheEntry.writeTime;
@@ -870,13 +913,13 @@ api.metadata = async function() {
 
 /******************************************************************************/
 
-api.purge = assetCacheMarkAsDirty;
+assets.purge = assetCacheMarkAsDirty;
 
-api.remove = function(pattern) {
+assets.remove = function(pattern) {
     return assetCacheRemove(pattern);
 };
 
-api.rmrf = function() {
+assets.rmrf = function() {
     return assetCacheRemove(/./);
 };
 
@@ -973,7 +1016,7 @@ const updateDone = function() {
     fireNotification('after-assets-updated', { assetKeys: assetKeys });
 };
 
-api.updateStart = function(details) {
+assets.updateStart = function(details) {
     const oldUpdateDelay = updaterAssetDelay;
     const newUpdateDelay = typeof details.delay === 'number'
         ? details.delay
@@ -990,7 +1033,7 @@ api.updateStart = function(details) {
     updateFirst();
 };
 
-api.updateStop = function() {
+assets.updateStop = function() {
     if ( updaterTimer ) {
         clearTimeout(updaterTimer);
         updaterTimer = undefined;
@@ -1000,17 +1043,13 @@ api.updateStop = function() {
     }
 };
 
-api.isUpdating = function() {
+assets.isUpdating = function() {
     return updaterStatus === 'updating' &&
-           updaterAssetDelay <= µBlock.hiddenSettings.manualUpdateAssetFetchPeriod;
+           updaterAssetDelay <= µb.hiddenSettings.manualUpdateAssetFetchPeriod;
 };
 
 /******************************************************************************/
 
-return api;
-
-/******************************************************************************/
-
-})();
+export default assets;
 
 /******************************************************************************/
