@@ -23,11 +23,17 @@
 
 /******************************************************************************/
 
-µBlock.cosmeticFilteringEngine = (( ) => {
+import './utils.js';
+import logger from './logger.js';
+import µb from './background.js';
+
+import {
+    StaticExtFilteringHostnameDB,
+    StaticExtFilteringSessionDB,
+} from './static-ext-filtering-db.js';
 
 /******************************************************************************/
 
-const µb = µBlock;
 const cosmeticSurveyingMissCountMax =
     parseInt(vAPI.localStorage.getItem('cosmeticSurveyingMissCountMax'), 10) ||
     15;
@@ -196,10 +202,10 @@ const FilterContainer = function() {
     this.selectorCacheTimer = null;
 
     // specific filters
-    this.specificFilters = new µb.staticExtFilteringEngine.HostnameBasedDB(2);
+    this.specificFilters = new StaticExtFilteringHostnameDB(2);
 
     // temporary filters
-    this.sessionFilterDB = new µb.staticExtFilteringEngine.SessionDB();
+    this.sessionFilterDB = new StaticExtFilteringSessionDB();
 
     // low generic cosmetic filters, organized by id/class then simple/complex.
     this.lowlyGeneric = Object.create(null);
@@ -250,7 +256,6 @@ const FilterContainer = function() {
 // Reset all, thus reducing to a minimum memory footprint of the context.
 
 FilterContainer.prototype.reset = function() {
-    this.µburi = µb.URI;
     this.frozen = false;
     this.acceptedCount = 0;
     this.discardedCount = 0;
@@ -340,16 +345,14 @@ FilterContainer.prototype.keyFromSelector = function(selector) {
 /******************************************************************************/
 
 FilterContainer.prototype.compile = function(parser, writer) {
-    writer.select(µb.compiledCosmeticSection);
-
     if ( parser.hasOptions() === false ) {
         this.compileGenericSelector(parser, writer);
         return true;
     }
 
     // https://github.com/chrisaljoudi/uBlock/issues/151
-    // Negated hostname means the filter applies to all non-negated hostnames
-    // of same filter OR globally if there is no non-negated hostnames.
+    //   Negated hostname means the filter applies to all non-negated hostnames
+    //   of same filter OR globally if there is no non-negated hostnames.
     let applyGlobally = true;
     for ( const { hn, not, bad } of parser.extOptions() ) {
         if ( bad ) { continue; }
@@ -381,16 +384,19 @@ FilterContainer.prototype.compileGenericHideSelector = function(
     parser,
     writer
 ) {
-    const { raw, compiled, pseudoclass } = parser.result;
+    const { raw, compiled } = parser.result;
     if ( compiled === undefined ) {
-        const who = writer.properties.get('assetKey') || '?';
-        µb.logger.writeOne({
+        const who = writer.properties.get('name') || '?';
+        logger.writeOne({
             realm: 'message',
             type: 'error',
             text: `Invalid generic cosmetic filter in ${who}: ${raw}`
         });
+        return;
     }
 
+    writer.select('COSMETIC_FILTERS:GENERIC');
+
     const type = compiled.charCodeAt(0);
     let key;
 
@@ -423,12 +429,12 @@ FilterContainer.prototype.compileGenericHideSelector = function(
     // https://github.com/uBlockOrigin/uBlock-issues/issues/131
     //   Support generic procedural filters as per advanced settings.
     //   TODO: prevent double compilation.
-    if ( compiled !== raw && pseudoclass === false ) {
+    if ( compiled !== raw ) {
         if ( µb.hiddenSettings.allowGenericProceduralFilters === true ) {
             return this.compileSpecificSelector(parser, '', false, writer);
         }
-        const who = writer.properties.get('assetKey') || '?';
-        µb.logger.writeOne({
+        const who = writer.properties.get('name') || '?';
+        logger.writeOne({
             realm: 'message',
             type: 'error',
             text: `Invalid generic cosmetic filter in ${who}: ##${raw}`
@@ -483,8 +489,8 @@ FilterContainer.prototype.compileGenericUnhideSelector = function(
     // Procedural cosmetic filters are acceptable as generic exception filters.
     const { raw, compiled } = parser.result;
     if ( compiled === undefined ) {
-        const who = writer.properties.get('assetKey') || '?';
-        µb.logger.writeOne({
+        const who = writer.properties.get('name') || '?';
+        logger.writeOne({
             realm: 'message',
             type: 'error',
             text: `Invalid cosmetic filter in ${who}: #@#${raw}`
@@ -492,6 +498,8 @@ FilterContainer.prototype.compileGenericUnhideSelector = function(
         return;
     }
 
+    writer.select('COSMETIC_FILTERS:SPECIFIC');
+
     // https://github.com/chrisaljoudi/uBlock/issues/497
     //   All generic exception filters are stored as hostname-based filter
     //   whereas the hostname is the empty string (which matches all
@@ -511,8 +519,8 @@ FilterContainer.prototype.compileSpecificSelector = function(
 ) {
     const { raw, compiled, exception } = parser.result;
     if ( compiled === undefined ) {
-        const who = writer.properties.get('assetKey') || '?';
-        µb.logger.writeOne({
+        const who = writer.properties.get('name') || '?';
+        logger.writeOne({
             realm: 'message',
             type: 'error',
             text: `Invalid cosmetic filter in ${who}: ##${raw}`
@@ -520,6 +528,8 @@ FilterContainer.prototype.compileSpecificSelector = function(
         return;
     }
 
+    writer.select('COSMETIC_FILTERS:SPECIFIC');
+
     // https://github.com/chrisaljoudi/uBlock/issues/145
     let unhide = exception ? 1 : 0;
     if ( not ) { unhide ^= 1; }
@@ -551,18 +561,13 @@ FilterContainer.prototype.compileTemporary = function(parser) {
 
 FilterContainer.prototype.fromCompiledContent = function(reader, options) {
     if ( options.skipCosmetic ) {
-        this.skipCompiledContent(reader);
-        return;
-    }
-    if ( options.skipGenericCosmetic ) {
-        this.skipGenericCompiledContent(reader);
+        this.skipCompiledContent(reader, 'SPECIFIC');
+        this.skipCompiledContent(reader, 'GENERIC');
         return;
     }
 
-    reader.select(µb.compiledCosmeticSection);
-
-    let db, bucket;
-
+    // Specific cosmetic filter section
+    reader.select('COSMETIC_FILTERS:SPECIFIC');
     while ( reader.next() ) {
         this.acceptedCount += 1;
         const fingerprint = reader.fingerprint();
@@ -571,56 +576,8 @@ FilterContainer.prototype.fromCompiledContent = function(reader, options) {
             continue;
         }
         this.duplicateBuster.add(fingerprint);
-
         const args = reader.args();
-
         switch ( args[0] ) {
-
-        // low generic, simple
-        case 0: // #AdBanner
-        case 2: // .largeAd
-            db = args[0] === 0 ? this.lowlyGeneric.id : this.lowlyGeneric.cl;
-            bucket = db.complex.get(args[1]);
-            if ( bucket === undefined ) {
-                db.simple.add(args[1]);
-            } else if ( Array.isArray(bucket) ) {
-                bucket.push(db.prefix + args[1]);
-            } else {
-                db.complex.set(args[1], [ bucket, db.prefix + args[1] ]);
-            }
-            break;
-
-        // low generic, complex
-        case 1: // #tads + div + .c
-        case 3: // .Mpopup + #Mad > #MadZone
-            db = args[0] === 1 ? this.lowlyGeneric.id : this.lowlyGeneric.cl;
-            bucket = db.complex.get(args[1]);
-            if ( bucket === undefined ) {
-                if ( db.simple.has(args[1]) ) {
-                    db.complex.set(args[1], [ db.prefix + args[1], args[2] ]);
-                } else {
-                    db.complex.set(args[1], args[2]);
-                    db.simple.add(args[1]);
-                }
-            } else if ( Array.isArray(bucket) ) {
-                bucket.push(args[2]);
-            } else {
-                db.complex.set(args[1], [ bucket, args[2] ]);
-            }
-            break;
-
-        // High-high generic hide/simple selectors
-        // div[id^="allo"]
-        case 4:
-            this.highlyGeneric.simple.dict.add(args[1]);
-            break;
-
-        // High-high generic hide/complex selectors
-        // div[id^="allo"] > span
-        case 5:
-            this.highlyGeneric.complex.dict.add(args[1]);
-            break;
-
         // hash,  example.com, .promoted-tweet
         // hash,  example.*, .promoted-tweet
         //
@@ -639,7 +596,72 @@ FilterContainer.prototype.fromCompiledContent = function(reader, options) {
             }
             this.specificFilters.store(args[1], args[2] & 0b011, args[3]);
             break;
+        default:
+            this.discardedCount += 1;
+            break;
+        }
+    }
 
+    if ( options.skipGenericCosmetic ) {
+        this.skipCompiledContent(reader, 'GENERIC');
+        return;
+    }
+
+    // Generic cosmetic filter section
+    reader.select('COSMETIC_FILTERS:GENERIC');
+    while ( reader.next() ) {
+        this.acceptedCount += 1;
+        const fingerprint = reader.fingerprint();
+        if ( this.duplicateBuster.has(fingerprint) ) {
+            this.discardedCount += 1;
+            continue;
+        }
+        this.duplicateBuster.add(fingerprint);
+        const args = reader.args();
+        switch ( args[0] ) {
+        // low generic, simple
+        case 0:   // #AdBanner
+        case 2: { // .largeAd
+            const db = args[0] === 0 ? this.lowlyGeneric.id : this.lowlyGeneric.cl;
+            const bucket = db.complex.get(args[1]);
+            if ( bucket === undefined ) {
+                db.simple.add(args[1]);
+            } else if ( Array.isArray(bucket) ) {
+                bucket.push(db.prefix + args[1]);
+            } else {
+                db.complex.set(args[1], [ bucket, db.prefix + args[1] ]);
+            }
+            break;
+        }
+        // low generic, complex
+        case 1:   // #tads + div + .c
+        case 3: { // .Mpopup + #Mad > #MadZone
+            const db = args[0] === 1 ? this.lowlyGeneric.id : this.lowlyGeneric.cl;
+            const bucket = db.complex.get(args[1]);
+            if ( bucket === undefined ) {
+                if ( db.simple.has(args[1]) ) {
+                    db.complex.set(args[1], [ db.prefix + args[1], args[2] ]);
+                } else {
+                    db.complex.set(args[1], args[2]);
+                    db.simple.add(args[1]);
+                }
+            } else if ( Array.isArray(bucket) ) {
+                bucket.push(args[2]);
+            } else {
+                db.complex.set(args[1], [ bucket, args[2] ]);
+            }
+            break;
+        }
+        // High-high generic hide/simple selectors
+        // div[id^="allo"]
+        case 4:
+            this.highlyGeneric.simple.dict.add(args[1]);
+            break;
+        // High-high generic hide/complex selectors
+        // div[id^="allo"] > span
+        case 5:
+            this.highlyGeneric.complex.dict.add(args[1]);
+            break;
         default:
             this.discardedCount += 1;
             break;
@@ -649,50 +671,8 @@ FilterContainer.prototype.fromCompiledContent = function(reader, options) {
 
 /******************************************************************************/
 
-FilterContainer.prototype.skipGenericCompiledContent = function(reader) {
-    reader.select(µb.compiledCosmeticSection);
-
-    while ( reader.next() ) {
-        this.acceptedCount += 1;
-        const fingerprint = reader.fingerprint();
-        if ( this.duplicateBuster.has(fingerprint) ) {
-            this.discardedCount += 1;
-            continue;
-        }
-
-        const args = reader.args();
-
-        switch ( args[0] ) {
-
-        // https://github.com/uBlockOrigin/uBlock-issues/issues/803
-        //   Handle specific filters meant to apply everywhere, i.e. selectors
-        //   not to be injected conditionally through the DOM surveyor.
-        //   hash,  *, .promoted-tweet
-        case 8:
-            this.duplicateBuster.add(fingerprint);
-            if ( args[2] === 0b100 ) {
-                if ( this.reSimpleHighGeneric.test(args[3]) )
-                    this.highlyGeneric.simple.dict.add(args[3]);
-                else {
-                    this.highlyGeneric.complex.dict.add(args[3]);
-                }
-                break;
-            }
-            this.specificFilters.store(args[1], args[2] & 0b011, args[3]);
-            break;
-
-        default:
-            this.discardedCount += 1;
-            break;
-        }
-   }
-};
-
-/******************************************************************************/
-
-FilterContainer.prototype.skipCompiledContent = function(reader) {
-    reader.select(µb.compiledCosmeticSection);
-
+FilterContainer.prototype.skipCompiledContent = function(reader, sectionId) {
+    reader.select(`COSMETIC_FILTERS:${sectionId}`);
     while ( reader.next() ) {
         this.acceptedCount += 1;
         this.discardedCount += 1;
@@ -847,6 +827,7 @@ FilterContainer.prototype.retrieveGenericSelectors = function(request) {
     if ( this.acceptedCount === 0 ) { return; }
     if ( !request.ids && !request.classes ) { return; }
 
+    const { safeOnly = false } = request;
     //console.time('cosmeticFilteringEngine.retrieveGenericSelectors');
 
     const simpleSelectors = this.$simpleSet;
@@ -859,25 +840,26 @@ FilterContainer.prototype.retrieveGenericSelectors = function(request) {
         const entry = this.lowlyGeneric[type];
         const selectors = request[entry.canonical];
         if ( Array.isArray(selectors) === false ) { continue; }
-        for ( let selector of selectors ) {
-            if ( entry.simple.has(selector) === false ) { continue; }
-            const bucket = entry.complex.get(selector);
-            if ( bucket !== undefined ) {
-                if ( Array.isArray(bucket) ) {
-                    for ( const selector of bucket ) {
-                        if ( previousHits.has(selector) === false ) {
-                            complexSelectors.add(selector);
-                        }
-                    }
-                } else if ( previousHits.has(bucket) === false ) {
-                    complexSelectors.add(bucket);
-                }
-            } else {
-                selector = entry.prefix + selector;
-                if ( previousHits.has(selector) === false ) {
-                    simpleSelectors.add(selector);
-                }
+        for ( const identifier of selectors ) {
+            if ( entry.simple.has(identifier) === false ) { continue; }
+            const bucket = entry.complex.get(identifier);
+            if ( typeof bucket === 'string' ) {
+                if ( previousHits.has(bucket) ) { continue; }
+                complexSelectors.add(bucket);
+                continue;
             }
+            const simpleSelector = entry.prefix + identifier;
+            if ( Array.isArray(bucket) ) {
+                for ( const complexSelector of bucket ) {
+                    if ( previousHits.has(complexSelector) ) { continue; }
+                    if ( safeOnly && complexSelector === simpleSelector ) { continue; }
+                    complexSelectors.add(complexSelector);
+                }
+                continue;
+            }
+            if ( previousHits.has(simpleSelector) ) { continue; }
+            if ( safeOnly ) { continue; }
+            simpleSelectors.add(simpleSelector);
         }
     }
 
@@ -1046,11 +1028,6 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
                         continue;
                     }
                 }
-                if ( pfilter.pseudo !== undefined ) {
-                    injectedHideFilters.push(pfilter.selector);
-                    proceduralSet.delete(json);
-                    continue;
-                }
             }
             if ( proceduralSet.size !== 0 ) {
                 out.proceduralFilters = Array.from(proceduralSet);
@@ -1149,47 +1126,39 @@ FilterContainer.prototype.getFilterCount = function() {
 
 /******************************************************************************/
 
-FilterContainer.prototype.benchmark = async function() {
-    const requests = await µb.loadBenchmarkDataset();
-    if ( Array.isArray(requests) === false || requests.length === 0 ) {
-        console.info('No requests found to benchmark');
-        return;
+FilterContainer.prototype.dump = function() {
+    let genericCount = 0;
+    for ( const i of [ 'simple', 'complex' ] ) {
+        for ( const j of [ 'id', 'cl' ] ) {
+            genericCount += this.lowlyGeneric[j][i].size;
+        }
     }
-    console.info('Benchmarking cosmeticFilteringEngine.retrieveSpecificSelectors()...');
-    const details = {
-        tabId: undefined,
-        frameId: undefined,
-        hostname: '',
-        domain: '',
-        entity: '',
-    };
-    const options = {
-        noSpecificCosmeticFiltering: false,
-        noGenericCosmeticFiltering: false,
-    };
-    let count = 0;
-    const t0 = self.performance.now();
-    for ( let i = 0; i < requests.length; i++ ) {
-        const request = requests[i];
-        if ( request.cpt !== 'main_frame' ) { continue; }
-        count += 1;
-        details.hostname = µb.URI.hostnameFromURI(request.url);
-        details.domain = µb.URI.domainFromHostname(details.hostname);
-        details.entity = µb.URI.entityFromDomain(details.domain);
-        void this.retrieveSpecificSelectors(details, options);
-    }
-    const t1 = self.performance.now();
-    const dur = t1 - t0;
-    console.info(`Evaluated ${count} requests in ${dur.toFixed(0)} ms`);
-    console.info(`\tAverage: ${(dur / count).toFixed(3)} ms per request`);
+    return [
+        'Cosmetic Filtering Engine internals:',
+        `specific: ${this.specificFilters.size}`,
+        `generic: ${genericCount}`,
+        `+ lowly.id: ${this.lowlyGeneric.id.simple.size + this.lowlyGeneric.id.complex.size}`,
+        `  + simple: ${this.lowlyGeneric.id.simple.size}`,
+        ...Array.from(this.lowlyGeneric.id.simple).map(a => `    ###${a}`),
+        `  + complex: ${this.lowlyGeneric.id.complex.size}`,
+        ...Array.from(this.lowlyGeneric.id.complex.values()).map(a => `    ##${a}`),
+        `+ lowly.class: ${this.lowlyGeneric.cl.simple.size + this.lowlyGeneric.cl.complex.size}`,
+        `  + simple: ${this.lowlyGeneric.cl.simple.size}`,
+        ...Array.from(this.lowlyGeneric.cl.simple).map(a => `    ##.${a}`),
+        `  + complex: ${this.lowlyGeneric.cl.complex.size}`,
+        ...Array.from(this.lowlyGeneric.cl.complex.values()).map(a => `    ##${a}`),
+        `+ highly: ${this.highlyGeneric.simple.dict.size + this.highlyGeneric.complex.dict.size}`,
+        `  + highly.simple: ${this.highlyGeneric.simple.dict.size}`,
+        ...Array.from(this.highlyGeneric.simple.dict).map(a => `    ##${a}`),
+        `  + highly.complex: ${this.highlyGeneric.complex.dict.size}`,
+        ...Array.from(this.highlyGeneric.complex.dict).map(a => `    ##${a}`),
+    ].join('\n');
 };
 
 /******************************************************************************/
 
-return new FilterContainer();
+const cosmeticFilteringEngine = new FilterContainer();
 
-/******************************************************************************/
-
-})();
+export default cosmeticFilteringEngine;
 
 /******************************************************************************/
