@@ -19,17 +19,20 @@
     Home: https://github.com/gorhill/uMatrix
 */
 
-/* global diff_match_patch, CodeMirror, uDom, uBlockDashboard */
+/* global CodeMirror, diff_match_patch, uDom, uBlockDashboard */
 
 'use strict';
 
 /******************************************************************************/
 
-(( ) => {
+import publicSuffixList from '../lib/publicsuffixlist/publicsuffixlist.js';
+
+import { hostnameFromURI } from './uri-utils.js';
+
+import './codemirror/ubo-dynamic-filtering.js';
 
 /******************************************************************************/
 
-const psl = self.publicSuffixList;
 const hostnameToDomainMap = new Map();
 
 const mergeView = new CodeMirror.MergeView(
@@ -393,7 +396,9 @@ const onPresentationChanged = (( ) => {
     const sortNormalizeHn = function(hn) {
         let domain = hostnameToDomainMap.get(hn);
         if ( domain === undefined ) {
-            domain = /(\d|\])$/.test(hn) ? hn : psl.getDomain(hn);
+            domain = /(\d|\])$/.test(hn)
+                ? hn
+                : publicSuffixList.getDomain(hn);
             hostnameToDomainMap.set(hn, domain);
         }
         let normalized = domain || hn;
@@ -424,7 +429,7 @@ const onPresentationChanged = (( ) => {
         } else if ( (match = reUrlRule.exec(rule)) !== null ) {
             type = '\x10FFFF';
             srcHn = sortNormalizeHn(match[1]);
-            desHn = sortNormalizeHn(vAPI.hostnameFromURI(match[2]));
+            desHn = sortNormalizeHn(hostnameFromURI(match[2]));
             extra = match[3];
         }
         if ( sortType === 0 ) {
@@ -499,13 +504,13 @@ const onPresentationChanged = (( ) => {
             const mode = origPane.doc.getMode();
             mode.sortType = sortType;
             mode.setHostnameToDomainMap(hostnameToDomainMap);
-            mode.setPSL(psl);
+            mode.setPSL(publicSuffixList);
         }
         {
             const mode = editPane.doc.getMode();
             mode.sortType = sortType;
             mode.setHostnameToDomainMap(hostnameToDomainMap);
-            mode.setPSL(psl);
+            mode.setPSL(publicSuffixList);
         }
         sort(origPane.modified);
         sort(editPane.modified);
@@ -643,7 +648,7 @@ vAPI.messaging.send('dashboard', {
 }).then(details => {
     thePanes.orig.original = details.permanentRules;
     thePanes.edit.original = details.sessionRules;
-    psl.fromSelfie(details.pslSelfie);
+    publicSuffixList.fromSelfie(details.pslSelfie);
     onPresentationChanged(true);
 });
 
@@ -668,5 +673,3 @@ mergeView.editor().on('updateDiff', ( ) => { onTextChanged(); });
 
 /******************************************************************************/
 
-})();
-
