@@ -23,9 +23,11 @@
 
 /******************************************************************************/
 
-µBlock.redirectEngine = (( ) => {
+import {
+    LineIterator,
+    orphanizeString,
+} from './text-utils.js';
 
-/******************************************************************************/
 /******************************************************************************/
 
 // The resources referenced below are found in ./web_accessible_resources/
@@ -78,6 +80,9 @@ const redirectableResources = new Map([
     [ 'empty', {
         data: 'text',   // Important!
     } ],
+    [ 'fingerprint2.js', {
+        data: 'text',
+    } ],
     [ 'google-analytics_analytics.js', {
         alias: [
             'google-analytics.com/analytics.js',
@@ -125,6 +130,9 @@ const redirectableResources = new Map([
         alias: 'bab-defuser.js',
         data: 'text',
     } ],
+    [ 'nobab2.js', {
+        data: 'text',
+    } ],
     [ 'nofab.js', {
         alias: 'fuckadblock.js-3.2.0',
         data: 'text',
@@ -200,6 +208,14 @@ const mimeFromName = function(name) {
     }
 };
 
+// vAPI.warSecret() is optional, it could be absent in some environments,
+// i.e. nodejs for example. Probably the best approach is to have the
+// "web_accessible_resources secret" added outside by the client of this
+// module, but for now I just want to remove an obstacle to modularization.
+const warSecret = typeof vAPI === 'object' && vAPI !== null
+    ? vAPI.warSecret
+    : ( ) => '';
+
 /******************************************************************************/
 /******************************************************************************/
 
@@ -226,14 +242,20 @@ const RedirectEntry = class {
             fctxt instanceof Object &&
             fctxt.type !== 'xmlhttprequest'
         ) {
-            let url = `${this.warURL}?secret=${vAPI.warSecret()}`;
+            const params = [];
+            const secret = warSecret();
+            if ( secret !== '' ) { params.push(`secret=${secret}`); }
             if ( this.params !== undefined ) {
                 for ( const name of this.params ) {
                     const value = fctxt[name];
                     if ( value === undefined ) { continue; }
-                    url += `&${name}=${encodeURIComponent(value)}`;
+                    params.push(`${name}=${encodeURIComponent(value)}`);
                 }
             }
+            let url = `${this.warURL}`;
+            if ( params.length !== 0 ) {
+                url += `?${params.join('&')}`;
+            }
             return url;
         }
         if ( this.data === undefined ) { return; }
@@ -356,7 +378,7 @@ RedirectEngine.prototype.resourceContentFromName = function(name, mime) {
 //   Append newlines to raw text to ensure processing of trailing resource.
 
 RedirectEngine.prototype.resourcesFromString = function(text) {
-    const lineIter = new µBlock.LineIterator(
+    const lineIter = new LineIterator(
         removeTopCommentBlock(text) + '\n\n'
     );
     const reNonEmptyLine = /\S/;
@@ -408,7 +430,7 @@ RedirectEngine.prototype.resourcesFromString = function(text) {
         // No more data, add the resource.
         const name = this.aliases.get(fields[0]) || fields[0];
         const mime = fields[1];
-        const content = µBlock.orphanizeString(
+        const content = orphanizeString(
             fields.slice(2).join(encoded ? '' : '\n')
         );
         this.resources.set(
@@ -435,21 +457,18 @@ const removeTopCommentBlock = function(text) {
 
 /******************************************************************************/
 
-RedirectEngine.prototype.loadBuiltinResources = function() {
-    // TODO: remove once usage of uBO 1.20.4 is widespread.
-    µBlock.assets.remove('ublock-resources');
-
+RedirectEngine.prototype.loadBuiltinResources = function(fetcher) {
     this.resources = new Map();
     this.aliases = new Map();
 
     const fetches = [
-        µBlock.assets.fetchText(
+        fetcher(
             '/assets/resources/scriptlets.js'
         ).then(result => {
             const content = result.content;
-            if ( typeof content === 'string' && content.length !== 0 ) {
-                this.resourcesFromString(content);
-            }
+            if ( typeof content !== 'string' ) { return; }
+            if ( content.length === 0 ) { return; }
+            this.resourcesFromString(content);
         }),
     ];
 
@@ -458,7 +477,7 @@ RedirectEngine.prototype.loadBuiltinResources = function() {
         const entry = RedirectEntry.fromSelfie({
             mime: mimeFromName(name),
             data,
-            warURL: vAPI.getURL(`/web_accessible_resources/${name}`),
+            warURL: `/web_accessible_resources/${name}`,
             params: details.params,
         });
         this.resources.set(name, entry);
@@ -505,10 +524,9 @@ RedirectEngine.prototype.loadBuiltinResources = function() {
             continue;
         }
         fetches.push(
-            µBlock.assets.fetch(
-                `/web_accessible_resources/${name}?secret=${vAPI.warSecret()}`,
-                { responseType: details.data }
-            ).then(
+            fetcher(`/web_accessible_resources/${name}`, {
+                responseType: details.data
+            }).then(
                 result => process(result)
             )
         );
@@ -544,21 +562,22 @@ RedirectEngine.prototype.getResourceDetails = function() {
 
 /******************************************************************************/
 
-const resourcesSelfieVersion = 5;
+const RESOURCES_SELFIE_VERSION = 6;
+const RESOURCES_SELFIE_NAME = 'compiled/redirectEngine/resources';
 
-RedirectEngine.prototype.selfieFromResources = function() {
-    µBlock.assets.put(
-        'compiled/redirectEngine/resources',
+RedirectEngine.prototype.selfieFromResources = function(storage) {
+    storage.put(
+        RESOURCES_SELFIE_NAME,
         JSON.stringify({
-            version: resourcesSelfieVersion,
+            version: RESOURCES_SELFIE_VERSION,
             aliases: Array.from(this.aliases),
             resources: Array.from(this.resources),
         })
     );
 };
 
-RedirectEngine.prototype.resourcesFromSelfie = async function() {
-    const result = await µBlock.assets.get('compiled/redirectEngine/resources');
+RedirectEngine.prototype.resourcesFromSelfie = async function(storage) {
+    const result = await storage.get(RESOURCES_SELFIE_NAME);
     let selfie;
     try {
         selfie = JSON.parse(result.content);
@@ -566,7 +585,7 @@ RedirectEngine.prototype.resourcesFromSelfie = async function() {
     }
     if (
         selfie instanceof Object === false ||
-        selfie.version !== resourcesSelfieVersion ||
+        selfie.version !== RESOURCES_SELFIE_VERSION ||
         Array.isArray(selfie.resources) === false
     ) {
         return false;
@@ -579,15 +598,14 @@ RedirectEngine.prototype.resourcesFromSelfie = async function() {
     return true;
 };
 
-RedirectEngine.prototype.invalidateResourcesSelfie = function() {
-    µBlock.assets.remove('compiled/redirectEngine/resources');
+RedirectEngine.prototype.invalidateResourcesSelfie = function(storage) {
+    storage.remove(RESOURCES_SELFIE_NAME);
 };
 
-/******************************************************************************/
 /******************************************************************************/
 
-return new RedirectEngine();
+const redirectEngine = new RedirectEngine();
+
+export { redirectEngine };
 
 /******************************************************************************/
-
-})();
