@@ -30,6 +30,12 @@
 
 /******************************************************************************/
 
+if ( typeof vAPI !== 'object' || vAPI === null ) {
+    return;
+}
+
+/******************************************************************************/
+
 const epickerId = vAPI.randomToken();
 let epickerConnectionId;
 
@@ -518,8 +524,19 @@ const filtersFrom = function(x, y) {
         x = undefined;
     }
 
-    // Network filter from element which was clicked.
-    if ( first !== null ) {
+    // https://github.com/gorhill/uBlock/issues/1545
+    //   Network filter candidates from all other elements found at [x,y].
+    // https://www.reddit.com/r/uBlockOrigin/comments/qmjk36/
+    //   Extract network candidates first.
+    if ( typeof x === 'number' ) {
+        const magicAttr = `${vAPI.sessionId}-clickblind`;
+        pickerRoot.setAttribute(magicAttr, '');
+        const elems = document.elementsFromPoint(x, y);
+        pickerRoot.removeAttribute(magicAttr);
+        for ( const elem of elems ) {
+            netFilterFromElement(elem);
+        }
+    } else if ( first !== null ) {
         netFilterFromElement(first);
     }
 
@@ -542,26 +559,6 @@ const filtersFrom = function(x, y) {
         }
     }
 
-    // https://github.com/gorhill/uBlock/issues/1545
-    //   Network filter candidates from all other elements found at
-    //   point (x, y).
-    if ( typeof x === 'number' ) {
-        const attrName = vAPI.sessionId + '-clickblind';
-        elem = first;
-        while ( elem !== null ) {
-            const previous = elem;
-            elem.setAttribute(attrName, '');
-            elem = elementFromPoint(x, y);
-            if ( elem === null || elem === previous ) { break; }
-            netFilterFromElement(elem);
-        }
-        for ( const elem of document.querySelectorAll(`[${attrName}]`) ) {
-            elem.removeAttribute(attrName);
-        }
-
-        netFilterFromElement(document.body);
-    }
-
     return netFilterCandidates.length + cosmeticFilterCandidates.length;
 };
 
