@@ -23,27 +23,59 @@
 
 /******************************************************************************/
 
+import './vapi-common.js';
+import './vapi-background.js';
+import './vapi-background-ext.js';
+
+// The following modules are loaded here until their content is better organized
+import './commands.js';
+import './messaging.js';
+import './storage.js';
+import './tab.js';
+import './ublock.js';
+import './utils.js';
+
+import cacheStorage from './cachestorage.js';
+import contextMenu from './contextmenu.js';
+import io from './assets.js';
+import lz4Codec from './lz4.js';
+import staticExtFilteringEngine from './static-ext-filtering.js';
+import staticFilteringReverseLookup from './reverselookup.js';
+import staticNetFilteringEngine from './static-net-filtering.js';
+import µb from './background.js';
+import webRequest from './traffic.js';
+import { redirectEngine } from './redirect-engine.js';
+import { ubolog } from './console.js';
+
+import {
+    permanentFirewall,
+    sessionFirewall,
+    permanentSwitches,
+    sessionSwitches,
+    permanentURLFiltering,
+    sessionURLFiltering,
+} from './filtering-engines.js';
+
+/******************************************************************************/
+
 // Load all: executed once.
 
 (async ( ) => {
 // >>>>> start of private scope
 
-const µb = µBlock;
-
 /******************************************************************************/
 
 vAPI.app.onShutdown = function() {
-    const µb = µBlock;
-    µb.staticFilteringReverseLookup.shutdown();
-    µb.assets.updateStop();
-    µb.staticNetFilteringEngine.reset();
-    µb.staticExtFilteringEngine.reset();
-    µb.sessionFirewall.reset();
-    µb.permanentFirewall.reset();
-    µb.sessionURLFiltering.reset();
-    µb.permanentURLFiltering.reset();
-    µb.sessionSwitches.reset();
-    µb.permanentSwitches.reset();
+    staticFilteringReverseLookup.shutdown();
+    io.updateStop();
+    staticNetFilteringEngine.reset();
+    staticExtFilteringEngine.reset();
+    sessionFirewall.reset();
+    permanentFirewall.reset();
+    sessionURLFiltering.reset();
+    permanentURLFiltering.reset();
+    sessionSwitches.reset();
+    permanentSwitches.reset();
 };
 
 /******************************************************************************/
@@ -113,7 +145,7 @@ const onVersionReady = function(lastVersion) {
 
     // Since built-in resources may have changed since last version, we
     // force a reload of all resources.
-    µb.redirectEngine.invalidateResourcesSelfie();
+    redirectEngine.invalidateResourcesSelfie(io);
 
     // https://github.com/LiCybora/NanoDefenderFirefox/issues/196
     //   Toggle on the blocking of CSP reports by default for Firefox.
@@ -121,8 +153,8 @@ const onVersionReady = function(lastVersion) {
         lastVersionInt <= 1031003011 &&
         vAPI.webextFlavor.soup.has('firefox')
     ) {
-        µb.sessionSwitches.toggle('no-csp-reports', '*', 1);
-        µb.permanentSwitches.toggle('no-csp-reports', '*', 1);
+        sessionSwitches.toggle('no-csp-reports', '*', 1);
+        permanentSwitches.toggle('no-csp-reports', '*', 1);
         µb.saveHostnameSwitches();
     }
 };
@@ -216,18 +248,71 @@ const onCacheSettingsReady = async function(fetched) {
     if ( fetched.compiledMagic !== µb.systemSettings.compiledMagic ) {
         µb.compiledFormatChanged = true;
         µb.selfieIsInvalid = true;
+        ubolog(`Serialized format of static filter lists changed`);
     }
     if ( fetched.selfieMagic !== µb.systemSettings.selfieMagic ) {
         µb.selfieIsInvalid = true;
+        ubolog(`Serialized format of selfie changed`);
     }
     if ( µb.selfieIsInvalid ) {
         µb.selfieManager.destroy();
-        µb.cacheStorage.set(µb.systemSettings);
+        cacheStorage.set(µb.systemSettings);
     }
 };
 
 /******************************************************************************/
 
+const onHiddenSettingsReady = async function() {
+    // Maybe customize webext flavor
+    if ( µb.hiddenSettings.modifyWebextFlavor !== 'unset' ) {
+        const tokens = µb.hiddenSettings.modifyWebextFlavor.split(/\s+/);
+        for ( const token of tokens ) {
+            switch ( token[0] ) {
+            case '+':
+                vAPI.webextFlavor.soup.add(token.slice(1));
+                break;
+            case '-':
+                vAPI.webextFlavor.soup.delete(token.slice(1));
+                break;
+            default:
+                vAPI.webextFlavor.soup.add(token);
+                break;
+            }
+        }
+        ubolog(`Override default webext flavor with ${tokens}`);
+    }
+
+    // Maybe override current network listener suspend state
+    if ( µb.hiddenSettings.suspendTabsUntilReady === 'no' ) {
+        vAPI.net.unsuspend(true);
+    } else if ( µb.hiddenSettings.suspendTabsUntilReady === 'yes' ) {
+        vAPI.net.suspend();
+    }
+
+    // Maybe disable WebAssembly
+    if ( vAPI.canWASM && µb.hiddenSettings.disableWebAssembly !== true ) {
+        const wasmModuleFetcher = function(path) {
+            return fetch(`${path}.wasm`, { mode: 'same-origin' }).then(
+                WebAssembly.compileStreaming
+            ).catch(reason => {
+                ubolog(reason);
+            });
+        };
+        staticNetFilteringEngine.enableWASM(wasmModuleFetcher, './js/wasm/').then(result => {
+            if ( result !== true ) { return; }
+            ubolog(`WASM modules ready ${Date.now()-vAPI.T0} ms after launch`);
+        });
+    }
+
+    // Maybe override default cache storage
+    const cacheBackend = await cacheStorage.select(
+        µb.hiddenSettings.cacheStorageAPI
+    );
+    ubolog(`Backend storage for cache will be ${cacheBackend}`);
+};
+
+/******************************************************************************/
+
 const onFirstFetchReady = function(fetched, adminExtra) {
     // https://github.com/uBlockOrigin/uBlock-issues/issues/507
     //   Firefox-specific: somehow `fetched` is undefined under certain
@@ -240,12 +325,12 @@ const onFirstFetchReady = function(fetched, adminExtra) {
     fromFetch(µb.localSettings, fetched);
     fromFetch(µb.restoreBackupSettings, fetched);
 
-    µb.permanentFirewall.fromString(fetched.dynamicFilteringString);
-    µb.sessionFirewall.assign(µb.permanentFirewall);
-    µb.permanentURLFiltering.fromString(fetched.urlFilteringString);
-    µb.sessionURLFiltering.assign(µb.permanentURLFiltering);
-    µb.permanentSwitches.fromString(fetched.hostnameSwitchesString);
-    µb.sessionSwitches.assign(µb.permanentSwitches);
+    permanentFirewall.fromString(fetched.dynamicFilteringString);
+    sessionFirewall.assign(permanentFirewall);
+    permanentURLFiltering.fromString(fetched.urlFilteringString);
+    sessionURLFiltering.assign(permanentURLFiltering);
+    permanentSwitches.fromString(fetched.hostnameSwitchesString);
+    sessionSwitches.assign(permanentSwitches);
 
     onNetWhitelistReady(fetched.netWhitelist, adminExtra);
     onVersionReady(fetched.version);
@@ -270,19 +355,9 @@ const fromFetch = function(to, fetched) {
 
 const createDefaultProps = function() {
     const fetchableProps = {
-        'dynamicFilteringString': [
-            'behind-the-scene * * noop',
-            'behind-the-scene * image noop',
-            'behind-the-scene * 3p noop',
-            'behind-the-scene * inline-script noop',
-            'behind-the-scene * 1p-script noop',
-            'behind-the-scene * 3p-script noop',
-            'behind-the-scene * 3p-frame noop',
-        ].join('\n'),
+        'dynamicFilteringString': µb.dynamicFilteringDefault.join('\n'),
         'urlFilteringString': '',
-        'hostnameSwitchesString': [
-            'no-large-media: behind-the-scene false',
-        ].join('\n'),
+        'hostnameSwitchesString': µb.hostnameSwitchesDefault.join('\n'),
         'lastRestoreFile': '',
         'lastRestoreTime': 0,
         'lastBackupFile': '',
@@ -290,10 +365,6 @@ const createDefaultProps = function() {
         'netWhitelist': µb.netWhitelistDefault,
         'version': '0.0.0.0'
     };
-    // https://github.com/LiCybora/NanoDefenderFirefox/issues/196
-    if ( vAPI.webextFlavor.soup.has('firefox') ) {
-        fetchableProps.hostnameSwitchesString += '\nno-csp-reports: * true';
-    }
     toFetch(µb.localSettings, fetchableProps);
     toFetch(µb.restoreBackupSettings, fetchableProps);
     return fetchableProps;
@@ -304,55 +375,38 @@ const createDefaultProps = function() {
 try {
     // https://github.com/gorhill/uBlock/issues/531
     await µb.restoreAdminSettings();
-    log.info(`Admin settings ready ${Date.now()-vAPI.T0} ms after launch`);
+    ubolog(`Admin settings ready ${Date.now()-vAPI.T0} ms after launch`);
 
     await µb.loadHiddenSettings();
-    log.info(`Hidden settings ready ${Date.now()-vAPI.T0} ms after launch`);
-
-    // Maybe override current network listener suspend state
-    if ( µb.hiddenSettings.suspendTabsUntilReady === 'no' ) {
-        vAPI.net.unsuspend(true);
-    } else if ( µb.hiddenSettings.suspendTabsUntilReady === 'yes' ) {
-        vAPI.net.suspend();
-    }
-
-    if ( µb.hiddenSettings.disableWebAssembly !== true ) {
-        µb.staticNetFilteringEngine.enableWASM().then(( ) => {
-            log.info(`WASM modules ready ${Date.now()-vAPI.T0} ms after launch`);
-        });
-    }
-
-    const cacheBackend = await µb.cacheStorage.select(
-        µb.hiddenSettings.cacheStorageAPI
-    );
-    log.info(`Backend storage for cache will be ${cacheBackend}`);
+    await onHiddenSettingsReady();
+    ubolog(`Hidden settings ready ${Date.now()-vAPI.T0} ms after launch`);
 
     const adminExtra = await vAPI.adminStorage.get('toAdd');
-    log.info(`Extra admin settings ready ${Date.now()-vAPI.T0} ms after launch`);
+    ubolog(`Extra admin settings ready ${Date.now()-vAPI.T0} ms after launch`);
 
     // https://github.com/uBlockOrigin/uBlock-issues/issues/1365
     //   Wait for onCacheSettingsReady() to be fully ready.
     const [ , , lastVersion ] = await Promise.all([
         µb.loadSelectedFilterLists().then(( ) => {
-            log.info(`List selection ready ${Date.now()-vAPI.T0} ms after launch`);
+            ubolog(`List selection ready ${Date.now()-vAPI.T0} ms after launch`);
         }),
-        µb.cacheStorage.get(
+        cacheStorage.get(
             { compiledMagic: 0, selfieMagic: 0 }
         ).then(fetched => {
-            log.info(`Cache magic numbers ready ${Date.now()-vAPI.T0} ms after launch`);
+            ubolog(`Cache magic numbers ready ${Date.now()-vAPI.T0} ms after launch`);
             onCacheSettingsReady(fetched);
         }),
         vAPI.storage.get(createDefaultProps()).then(fetched => {
-            log.info(`First fetch ready ${Date.now()-vAPI.T0} ms after launch`);
+            ubolog(`First fetch ready ${Date.now()-vAPI.T0} ms after launch`);
             onFirstFetchReady(fetched, adminExtra);
             return fetched.version;
         }),
         µb.loadUserSettings().then(fetched => {
-            log.info(`User settings ready ${Date.now()-vAPI.T0} ms after launch`);
+            ubolog(`User settings ready ${Date.now()-vAPI.T0} ms after launch`);
             onUserSettingsReady(fetched);
         }),
         µb.loadPublicSuffixList().then(( ) => {
-            log.info(`PSL ready ${Date.now()-vAPI.T0} ms after launch`);
+            ubolog(`PSL ready ${Date.now()-vAPI.T0} ms after launch`);
         }),
     ]);
 
@@ -366,7 +420,7 @@ try {
 }
 
 // Prime the filtering engines before first use.
-µb.staticNetFilteringEngine.prime();
+staticNetFilteringEngine.prime();
 
 // https://github.com/uBlockOrigin/uBlock-issues/issues/817#issuecomment-565730122
 //   Still try to load filter lists regardless of whether a serious error
@@ -375,7 +429,7 @@ let selfieIsValid = false;
 try {
     selfieIsValid = await µb.selfieManager.load();
     if ( selfieIsValid === true ) {
-        log.info(`Selfie ready ${Date.now()-vAPI.T0} ms after launch`);
+        ubolog(`Selfie ready ${Date.now()-vAPI.T0} ms after launch`);
     }
 } catch (ex) {
     console.trace(ex);
@@ -383,7 +437,7 @@ try {
 if ( selfieIsValid !== true ) {
     try {
         await µb.loadFilterLists();
-        log.info(`Filter lists ready ${Date.now()-vAPI.T0} ms after launch`);
+        ubolog(`Filter lists ready ${Date.now()-vAPI.T0} ms after launch`);
     } catch (ex) {
         console.trace(ex);
     }
@@ -396,21 +450,21 @@ if ( selfieIsValid !== true ) {
 µb.readyToFilter = true;
 
 // Start network observers.
-µb.webRequest.start();
+webRequest.start();
 
 // Ensure that the resources allocated for decompression purpose (likely
 // large buffers) are garbage-collectable immediately after launch.
 // Otherwise I have observed that it may take quite a while before the
 // garbage collection of these resources kicks in. Relinquishing as soon
 // as possible ensure minimal memory usage baseline.
-µb.lz4Codec.relinquish();
+lz4Codec.relinquish();
 
 // Initialize internal state with maybe already existing tabs.
 initializeTabs();
 
 // https://github.com/chrisaljoudi/uBlock/issues/184
 //   Check for updates not too far in the future.
-µb.assets.addObserver(µb.assetObserver.bind(µb));
+io.addObserver(µb.assetObserver.bind(µb));
 µb.scheduleAssetUpdater(
     µb.userSettings.autoUpdate
         ? µb.hiddenSettings.autoUpdateDelayAfterLaunch * 1000
@@ -419,7 +473,7 @@ initializeTabs();
 
 // Force an update of the context menu according to the currently
 // active tab.
-µb.contextMenu.update();
+contextMenu.update();
 
 // Maybe install non-default popup document, or automatically select
 // default UI according to platform.
@@ -445,14 +499,18 @@ if (
 browser.runtime.onUpdateAvailable.addListener(details => {
     const toInt = vAPI.app.intFromVersion;
     if (
-        µBlock.hiddenSettings.extensionUpdateForceReload === true ||
+        µb.hiddenSettings.extensionUpdateForceReload === true ||
         toInt(details.version) <= toInt(vAPI.app.version)
     ) {
         vAPI.app.restart();
     }
 });
 
-log.info(`All ready ${Date.now()-vAPI.T0} ms after launch`);
+µb.supportStats.launchToReadiness = `${Date.now() - vAPI.T0} ms`;
+if ( selfieIsValid ) {
+    µb.supportStats.launchToReadiness += ' (selfie)';
+}
+ubolog(`All ready ${µb.supportStats.launchToReadiness} ms after launch`);
 
 // <<<<< end of private scope
 })();
