@@ -70,7 +70,13 @@ CodeMirror.defineMode("ubo-whitelist-directives", function() {
                 }
                 return null;
             }
-            return reHostnameExtractor.test(line) ? null : 'error';
+            if ( reHostnameExtractor.test(line) === false ) {
+                return 'error';
+            }
+            if ( whitelistDefaultSet.has(line.trim()) ) {
+                return 'keyword';
+            }
+            return null;
         }
     };
 });
@@ -92,7 +98,7 @@ const cmEditor = new CodeMirror(
         autofocus: true,
         lineNumbers: true,
         lineWrapping: true,
-        styleActiveLine: true
+        styleActiveLine: true,
     }
 );
 
@@ -100,10 +106,21 @@ uBlockDashboard.patchCodeMirrorEditor(cmEditor);
 
 /******************************************************************************/
 
+const getEditorText = function() {
+    let text = cmEditor.getValue().replace(/\s+$/, '');
+    return text === '' ? text : text + '\n';
+};
+
+const setEditorText = function(text) {
+    cmEditor.setValue(text.replace(/\s+$/, '') + '\n');
+};
+
+/******************************************************************************/
+
 const whitelistChanged = function() {
     const whitelistElem = uDom.nodeFromId('whitelist');
     const bad = whitelistElem.querySelector('.cm-error') !== null;
-    const changedWhitelist = cmEditor.getValue().trim();
+    const changedWhitelist = getEditorText().trim();
     const changed = changedWhitelist !== cachedWhitelist;
     uDom.nodeFromId('whitelistApply').disabled = !changed || bad;
     uDom.nodeFromId('whitelistRevert').disabled = !changed;
@@ -144,12 +161,9 @@ const renderWhitelist = async function() {
         }
         return ad.localeCompare(bd);
     });
-    let whitelistStr = details.whitelist.join('\n').trim();
+    const whitelistStr = details.whitelist.join('\n').trim();
     cachedWhitelist = whitelistStr;
-    if ( whitelistStr !== '' ) {
-        whitelistStr += '\n';
-    }
-    cmEditor.setValue(whitelistStr);
+    setEditorText(whitelistStr);
     if ( first ) {
         cmEditor.clearHistory();
     }
@@ -164,11 +178,8 @@ const handleImportFilePicker = function() {
     const fr = new FileReader();
     fr.onload = ev => {
         if ( ev.type !== 'load' ) { return; }
-        cmEditor.setValue(
-            [
-                cmEditor.getValue().trim(),
-                fr.result.trim()
-            ].join('\n').trim()
+        setEditorText(
+            [ getEditorText().trim(), fr.result.trim() ].join('\n').trim()
         );
     };
     fr.readAsText(file);
@@ -188,7 +199,7 @@ const startImportFilePicker = function() {
 /******************************************************************************/
 
 const exportWhitelistToFile = function() {
-    const val = cmEditor.getValue().trim();
+    const val = getEditorText();
     if ( val === '' ) { return; }
     const filename =
         vAPI.i18n('whitelistExportFilename')
@@ -203,7 +214,7 @@ const exportWhitelistToFile = function() {
 /******************************************************************************/
 
 const applyChanges = async function() {
-    cachedWhitelist = cmEditor.getValue().trim();
+    cachedWhitelist = getEditorText().trim();
     await messaging.send('dashboard', {
         what: 'setWhitelist',
         whitelist: cachedWhitelist,
@@ -212,23 +223,21 @@ const applyChanges = async function() {
 };
 
 const revertChanges = function() {
-    let content = cachedWhitelist;
-    if ( content !== '' ) { content += '\n'; }
-    cmEditor.setValue(content);
+    setEditorText(cachedWhitelist);
 };
 
 /******************************************************************************/
 
 const getCloudData = function() {
-    return cmEditor.getValue();
+    return getEditorText();
 };
 
 const setCloudData = function(data, append) {
     if ( typeof data !== 'string' ) { return; }
     if ( append ) {
-        data = uBlockDashboard.mergeNewLines(cmEditor.getValue().trim(), data);
+        data = uBlockDashboard.mergeNewLines(getEditorText().trim(), data);
     }
-    cmEditor.setValue(data.trim() + '\n');
+    setEditorText(data.trim());
 };
 
 self.cloud.onPush = getCloudData;
@@ -237,7 +246,7 @@ self.cloud.onPull = setCloudData;
 /******************************************************************************/
 
 self.hasUnsavedData = function() {
-    return cmEditor.getValue().trim() !== cachedWhitelist;
+    return getEditorText().trim() !== cachedWhitelist;
 };
 
 /******************************************************************************/
