@@ -13,8 +13,9 @@
 
 /*! Home: https://github.com/gorhill/publicsuffixlist.js -- GPLv3 APLv2 */
 
-/* jshint browser:true, esversion:6, laxbreak:true, undef:true, unused:true */
-/* globals WebAssembly, console, exports:true, module */
+/* globals WebAssembly, exports:true, module */
+
+'use strict';
 
 /*******************************************************************************
 
@@ -42,11 +43,9 @@
 
 /******************************************************************************/
 
-(function(context) {
+export default (function() {
 // >>>>>>>> start of anonymous namespace
 
-'use strict';
-
 /*******************************************************************************
 
     Tree encoding in array buffer:
@@ -65,12 +64,13 @@
 */
 
                                     // i32 /  i8
-const HOSTNAME_SLOT       = 0;      // jshint ignore:line
-const LABEL_INDICES_SLOT  = 256;    //  -- / 256
-const RULES_PTR_SLOT      = 100;    // 100 / 400
-const CHARDATA_PTR_SLOT   = 101;    // 101 / 404
-const EMPTY_STRING        = '';
-const SELFIE_MAGIC        = 2;
+const HOSTNAME_SLOT         = 0;    // jshint ignore:line
+const LABEL_INDICES_SLOT    = 256;  //  -- / 256 (256/2 => 128 labels max)
+const RULES_PTR_SLOT        = 100;  // 100 / 400 (400-256=144 => 144>128)
+const SUFFIX_NOT_FOUND_SLOT = 399;  //  -- / 399 (safe, see above)
+const CHARDATA_PTR_SLOT     = 101;  // 101 / 404
+const EMPTY_STRING          = '';
+const SELFIE_MAGIC          = 2;
 
 let wasmMemory;
 let pslBuffer32;
@@ -81,13 +81,11 @@ let hostnameArg = EMPTY_STRING;
 /******************************************************************************/
 
 const fireChangedEvent = function() {
-    if (
-        window instanceof Object &&
-        window.dispatchEvent instanceof Function &&
-        window.CustomEvent instanceof Function
-    ) {
-        window.dispatchEvent(new CustomEvent('publicSuffixListChanged'));
-    }
+    if ( typeof window !== 'object' ) { return; }
+    if ( window instanceof Object === false ) { return; }
+    if ( window.dispatchEvent instanceof Function === false ) { return; }
+    if ( window.CustomEvent instanceof Function === false ) { return; }
+    window.dispatchEvent(new CustomEvent('publicSuffixListChanged'));
 };
 
 /******************************************************************************/
@@ -113,7 +111,7 @@ const allocateBuffers = function(byteLength) {
         pslBuffer8 = new Uint8Array(pslByteLength);
         pslBuffer32 = new Uint32Array(pslBuffer8.buffer);
     }
-    hostnameArg = '';
+    hostnameArg = EMPTY_STRING;
     pslBuffer8[LABEL_INDICES_SLOT] = 0;
 };
 
@@ -331,6 +329,7 @@ const setHostnameArg = function(hostname) {
     const buf = pslBuffer8;
     if ( hostname === hostnameArg ) { return buf[LABEL_INDICES_SLOT]; }
     if ( hostname === null || hostname.length === 0 ) {
+        hostnameArg = EMPTY_STRING;
         return (buf[LABEL_INDICES_SLOT] = 0);
     }
     hostname = hostname.toLowerCase();
@@ -405,6 +404,7 @@ const getPublicSuffixPosJS = function() {
         // 2. If no rules match, the prevailing rule is "*".
         if ( iFound === 0 ) {
             if ( buf8[iCandidates + 1 << 2] !== 0x2A /* '*' */ ) { break; }
+            buf8[SUFFIX_NOT_FOUND_SLOT] = 1;
             iFound = iCandidates;
         }
         iNode = iFound;
@@ -473,6 +473,24 @@ const getDomain = function(hostname) {
 
 /******************************************************************************/
 
+const suffixInPSL = function(hostname) {
+    if ( pslBuffer32 === undefined ) { return false; }
+
+    const hostnameLen = setHostnameArg(hostname);
+    const buf8 = pslBuffer8;
+    if ( hostnameLen === 0 || buf8[0] === 0x2E /* '.' */ ) {
+        return false;
+    }
+
+    buf8[SUFFIX_NOT_FOUND_SLOT] = 0;
+    const cursorPos = getPublicSuffixPos();
+    return cursorPos !== -1 &&
+           buf8[cursorPos + 1] === 0 &&
+           buf8[SUFFIX_NOT_FOUND_SLOT] !== 1;
+};
+
+/******************************************************************************/
+
 const toSelfie = function(encoder) {
     if ( pslBuffer8 === undefined ) { return ''; }
     if ( encoder instanceof Object ) {
@@ -516,7 +534,7 @@ const fromSelfie = function(selfie, decoder) {
     }
 
     // Important!
-    hostnameArg = '';
+    hostnameArg = EMPTY_STRING;
     pslBuffer8[LABEL_INDICES_SLOT] = 0;
 
     fireChangedEvent();
@@ -529,57 +547,34 @@ const fromSelfie = function(selfie, decoder) {
 // The WASM module is entirely optional, the JS implementation will be
 // used should the WASM module be unavailable for whatever reason.
 
-const enableWASM = (function() {
-    // The directory from which the current script was fetched should also
-    // contain the related WASM file. The script is fetched from a trusted
-    // location, and consequently so will be the related WASM file.
-    let workingDir;
-    {
-        const url = new URL(document.currentScript.src);
-        const match = /[^\/]+$/.exec(url.pathname);
-        if ( match !== null ) {
-            url.pathname = url.pathname.slice(0, match.index);
-        }
-        workingDir = url.href;
-    }
-
-    let memory;
-
-    return function() {
-        if ( getPublicSuffixPosWASM instanceof Function ) {
-            return Promise.resolve(true);
-        }
-
-        if (
-            typeof WebAssembly !== 'object' ||
-            typeof WebAssembly.instantiateStreaming !== 'function'
-        ) {
-            return Promise.resolve(false);
-        }
+const enableWASM = (( ) => {
+    let wasmPromise;
 
+    const getWasmInstance = async function(wasmModuleFetcher, path) {
+        if ( typeof WebAssembly !== 'object' ) { return false; }
         // The wasm code will work only if CPU is natively little-endian,
         // as we use native uint32 array in our js code.
         const uint32s = new Uint32Array(1);
         const uint8s = new Uint8Array(uint32s.buffer);
         uint32s[0] = 1;
-        if ( uint8s[0] !== 1 ) {
-            return Promise.resolve(false);
-        }
+        if ( uint8s[0] !== 1 ) { return false; }
 
-        return fetch(
-            workingDir + 'wasm/publicsuffixlist.wasm',
-            { mode: 'same-origin' }
-        ).then(response => {
+        try {
+            const module = await wasmModuleFetcher(`${path}publicsuffixlist`);
+            if (  module instanceof WebAssembly.Module === false ) {
+                return false;
+            }
             const pageCount = pslBuffer8 !== undefined
                 ? pslBuffer8.byteLength + 0xFFFF >>> 16
                 : 1;
-            memory = new WebAssembly.Memory({ initial: pageCount });
-            return WebAssembly.instantiateStreaming(
-                response,
-                { imports: { memory: memory } }
-            );
-        }).then(({ instance }) => {
-            const curPageCount = memory.buffer.byteLength;
+            const memory = new WebAssembly.Memory({ initial: pageCount });
+            const instance = await WebAssembly.instantiate(module, {
+                imports: { memory }
+            });
+            if (  instance instanceof WebAssembly.Instance === false ) {
+                return false;
+            }
+            const curPageCount = memory.buffer.byteLength >>> 16;
             const newPageCount = pslBuffer8 !== undefined
                 ? pslBuffer8.byteLength + 0xFFFF >>> 16
                 : 0;
@@ -596,12 +591,19 @@ const enableWASM = (function() {
             wasmMemory = memory;
             getPublicSuffixPosWASM = instance.exports.getPublicSuffixPos;
             getPublicSuffixPos = getPublicSuffixPosWASM;
-            memory = undefined;
             return true;
-        }).catch(reason => {
+        } catch(reason) {
             console.info(reason);
-            return false;
-        });
+        }
+        return false;
+    };
+
+    return async function(wasmModuleFetcher, path) {
+        if ( getPublicSuffixPosWASM instanceof Function ) { return true; }
+        if ( wasmPromise instanceof Promise === false ) {
+            wasmPromise = getWasmInstance(wasmModuleFetcher, path);
+        }
+        return wasmPromise;
     };
 })();
 
@@ -623,24 +625,17 @@ const disableWASM = function() {
 
 /******************************************************************************/
 
-context = context || window;
-
-context.publicSuffixList = {
+return ({
     version: '2.0',
     parse,
     getDomain,
+    suffixInPSL,
     getPublicSuffix,
     toSelfie, fromSelfie,
     disableWASM, enableWASM,
-};
-
-if ( typeof module !== 'undefined' ) { 
-    module.exports = context.publicSuffixList;
-} else if ( typeof exports !== 'undefined' ) {
-    exports = context.publicSuffixList;
-}
+});
 
 /******************************************************************************/
 
 // <<<<<<<< end of anonymous namespace
-})(this);
+})();
