@@ -7,21 +7,7 @@
 *   https://github.com/foo123/RegexAnalyzer
 *
 **/
-!function( root, name, factory ){
-"use strict";
-if ( ('undefined'!==typeof Components)&&('object'===typeof Components.classes)&&('object'===typeof Components.classesByID)&&Components.utils&&('function'===typeof Components.utils['import']) ) /* XPCOM */
-    (root.$deps = root.$deps||{}) && (root.EXPORTED_SYMBOLS = [name]) && (root[name] = root.$deps[name] = factory.call(root));
-else if ( ('object'===typeof module)&&module.exports ) /* CommonJS */
-    (module.$deps = module.$deps||{}) && (module.exports = module.$deps[name] = factory.call(root));
-else if ( ('undefined'!==typeof System)&&('function'===typeof System.register)&&('function'===typeof System['import']) ) /* ES6 module */
-    System.register(name,[],function($__export){$__export(name, factory.call(root));});
-else if ( ('function'===typeof define)&&define.amd&&('function'===typeof require)&&('function'===typeof require.specified)&&require.specified(name) /*&& !require.defined(name)*/ ) /* AMD */
-    define(name,['module'],function(module){factory.moduleUri = module.uri; return factory.call(root);});
-else if ( !(name in root) ) /* Browser/WebWorker/.. */
-    (root[name] = factory.call(root)||1)&&('function'===typeof(define))&&define.amd&&define(function(){return root[name];} );
-}(  /* current root */          'undefined' !== typeof self ? self : this,
-    /* module name */           "Regex",
-    /* module factory */        function ModuleFactory__Regex( undef ){
+export default (function () {
 "use strict";
 var __version__ = "1.1.0",
 
@@ -2153,4 +2139,4 @@ var Regex = {
 };
 /* export the module */
 return Regex;
-});
+})();
