@@ -2,10 +2,12 @@
 #
 # This script assumes a linux environment
 
+set -e
+
 echo "*** uBlock: Importing from Crowdin archive"
 
 SRC=~/Downloads/crowdin
-rm -r $SRC
+rm -r $SRC || true
 unzip -q ~/Downloads/uBlock\ \(translations\).zip -d $SRC
 
 DES=./src/_locales
@@ -58,6 +60,7 @@ cp $SRC/ro/messages.json    $DES/ro/messages.json
 cp $SRC/ru/messages.json    $DES/ru/messages.json
 cp $SRC/sk/messages.json    $DES/sk/messages.json
 cp $SRC/sl/messages.json    $DES/sl/messages.json
+cp $SRC/so/messages.json    $DES/so/messages.json
 cp $SRC/sq/messages.json    $DES/sq/messages.json
 cp $SRC/sr/messages.json    $DES/sr/messages.json
 cp $SRC/sv-SE/messages.json $DES/sv/messages.json
@@ -76,7 +79,7 @@ cp $SRC/zh-TW/messages.json $DES/zh_TW/messages.json
 # uBO does not use `$`, so any instance of `$` must be investigated.
 # See https://issues.adblockplus.org/ticket/6666
 echo "*** uBlock: Instances of '\$':"
-grep -FR "$" $DES/
+grep -FR "$" $DES/ || true
 
 #
 
