@@ -12,8 +12,6 @@ assets=(
     ['thirdparties/easylist-downloads.adblockplus.org/easyprivacy.txt']='https://easylist.to/easylist/easyprivacy.txt'
     ['thirdparties/pgl.yoyo.org/as/serverlist']='https://pgl.yoyo.org/adservers/serverlist.php?hostformat=hosts&showintro=1&startdate%5Bday%5D=&startdate%5Bmonth%5D=&startdate%5Byear%5D=&mimetype=plaintext'
     ['thirdparties/publicsuffix.org/list/effective_tld_names.dat']='https://publicsuffix.org/list/public_suffix_list.dat'
-    ['thirdparties/someonewhocares.org/hosts/hosts.txt']='http://someonewhocares.org/hosts/hosts'
-    ['thirdparties/winhelp2002.mvps.org/hosts.txt']='http://winhelp2002.mvps.org/hosts.txt'
     ['thirdparties/urlhaus-filter/urlhaus-filter-online.txt']='https://gitlab.com/curben/urlhaus-filter/raw/master/urlhaus-filter-online.txt'
 )
 
